<?php

/**
 * @file
 * Contains \Drupal\warden\Service\WardenManager.
 */

namespace Drupal\warden\Service;

use Drupal\Core\Extension\ExtensionDiscovery;
use Drupal\Core\Extension\InfoParser;
use WardenApi\Api;

/**
 * Default controller for the warden module.
 */
class WardenManager extends Api {

  /**
   * @var InfoParser
   */
  protected $infoParser;

  /**
   * WardenManager constructor.
   */
  public function __construct() {
    // @todo can we pass these variables in from the warden.services.yml?
    $url = \Drupal::config('warden.settings')->get('warden_server_host_path');
    $username = \Drupal::config('warden.settings')->get('warden_http_username');
    $password = \Drupal::config('warden.settings')->get('warden_http_password');
    parent::__construct($url, $username, $password);
  }

  /**
   * Generate all the site's data for Warden.
   *
   * @return array
   *   The site's data as an array.
   */
  public function generateSiteData() {
    global $base_url;

    $result = [
      'core' => [
        'drupal' => [
          'version' => \Drupal::VERSION,
        ],
      ],
      'contrib' => [],
      'custom' => [],
      'url' => $base_url,
      'site_name' =>  \Drupal::config('system.site')->get('name'),
      'key' => $this->getLocalToken(),
      'time' => time(),
    ];

    // Indicate if we are not including custom modules.
    if (!$this->includeCustomModules()) {
      $result['custom'] = 'disabled';
    }

    // Indicate if we are not including contrib modules.
    if (!$this->includeContribModules()) {
      $result['contrib'] = 'disabled';
    }

    /** @var \Drupal\Core\Extension\Extension[] $system_themes */
    $system_themes = system_list('theme');

    // Include all contrib themes.
    if ($this->includeContribModules()) {
      foreach ($system_themes as $theme) {
        if (isset($theme->info['package']) && $theme->info['package'] == 'Core') {
          continue;
        }

        if (isset($theme->info['version'])) {
          $result['contrib'][$theme->name] = [
            'version' => $theme->info['version'],
          ];
        }
      }
    }

    $listing = new ExtensionDiscovery(\Drupal::root());
    $system_modules = $listing->scan('module');

    // Include all modules.
    foreach ($system_modules as $module => $module_info) {
      $filename = $module_info->getPathname();

      // Match for custom modules.
      if ($this->includeCustomModules() && preg_match($this->getCustomRegex(), $filename)) {
        $result['custom'] = array_merge($result['custom'], $this->getModuleDetails($module, $filename));
      }

      // Match for contrib modules.
      if ($this->includeContribModules() && preg_match($this->getContribRegex(), $filename)) {
        $result['contrib'] = array_merge($result['contrib'], $this->getModuleDetails($module, $filename));
      }
    }

    return $result;
  }

  /**
   * Update Warden with latest site data.
   *
   * @throws \Exception
   *   If any problems occur.
   */
  public function updateWarden() {
    $data = $this->generateSiteData();
    $this->postSiteData($data);
  }

  /**
   * Get the local token or generate it if it is not set.
   *
   * @return string
   *   The local token
   */
  public function getLocalToken() {
    $local_token = \Drupal::config('warden.settings')->get('warden_token');

    if (empty($local_token)) {
      $local_token = hash('sha256', mt_rand());
      \Drupal::configFactory()
        ->getEditable('warden.settings')
        ->set('warden_token', $local_token)
        ->save();
    }

    return $local_token;
  }

  /**
   * @return string
   *   The regex telling us where a contrib module is located.
   */
  protected function getContribRegex() {
    return \Drupal::config('warden.settings')->get('warden_preg_match_contrib');
  }

  /**
   * @return string
   *   The regex telling us where a custom module is located.
   */
  protected function getCustomRegex() {
    return \Drupal::config('warden.settings')->get('warden_preg_match_custom');
  }

  /**
   * @return bool
   */
  protected function includeCustomModules() {
    $include_custom = &drupal_static(__FUNCTION__);

    if (is_null($include_custom)) {
      $include_custom = \Drupal::config('warden.settings')->get('warden_match_custom');
    }

    return $include_custom;
  }

  /**
   * @return bool
   */
  protected function includeContribModules() {
    $include_contrib = &drupal_static(__FUNCTION__);

    if (is_null($include_contrib)) {
      $include_contrib = \Drupal::config('warden.settings')->get('warden_match_contrib');
    }

    return $include_contrib;
  }

  /**
   * @return \Drupal\Core\Extension\InfoParser
   */
  protected function getInfoParser() {
    if (empty($this->infoParser)) {
      $this->infoParser = new InfoParser();
    }

    return $this->infoParser;
  }

  /**
   * @param string $filename
   *   A full path to an info yml file.
   *
   * @return array
   *   key is a project name or module name
   *   value is an array of its details
   */
  protected function getModuleDetails($module, $filename) {
    $details = $this->getInfoParser()->parse($filename);
    if (!empty($details['project'])) {
      return [
        $details['project'] => [
          'version' => $details['version'],
        ]
      ];
    }
    else {
      return [
        $module => [
          'version' => $details['version'],
        ]
      ];
    }
  }
}
